/// <reference types="react" />
import * as React from "react";
import { IFormBinder, IDataBinder, IFormBinderInjector } from "./formCore";
import { IValueConverter, IInputValueConverter, INumericOptions } from "./formValueConverters";
import { FormBinderBase } from "./formBinderBase";
import { ICalendarInputProps } from "./inputs/calendarInput";
import { IDateInputProps } from "./inputs/dateInput";
import { ITimeInputProps } from "./inputs/timeInput";
import { IAutoCompleteInputProps, IAutoCompleteOption } from "./inputs/autoCompleteInput";
/** An input FormBinder that sets native 'value' and 'onChange: (e) => void' properties */
export declare class InputFormBinder<TDataPropValue, TComponentPropValue> extends FormBinderBase<React.DOMAttributes<{}>, TDataPropValue, TComponentPropValue> {
    setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>): void;
    protected getDefaultInputValue(): any;
    handleValueChanged(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
export declare class SelectMultipleFormBinder<TDataProp> extends InputFormBinder<TDataProp, string[]> {
    constructor(dataPath: string, valueConverter?: IValueConverter<TDataProp, string[]>);
    setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>): void;
    handleValueChanged(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
    private getSelectValues(select);
}
export declare class CheckboxFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue> {
    constructor(dataPath: string, valueConverter?: IValueConverter<TDataPropValue, TComponentPropValue>);
    protected getDefaultInputValue(): boolean;
}
/** A radio input FormBinder */
export declare class RadioFormBinder<TDataPropValue, TComponentPropValue> extends InputFormBinder<TDataPropValue, TComponentPropValue> {
    setElementProperty(props: React.DOMAttributes<any>, dataBinder: IDataBinder<any>): void;
}
export declare class DateInputFormBinder extends FormBinderBase<IDateInputProps, string, string> {
    constructor(dataPath: string);
    handleValueChanged(props: IDateInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
export declare class TimeInputFormBinder extends FormBinderBase<ITimeInputProps, string, string> {
    constructor(dataPath: string);
    static customValue(dataName: string): TimeInputFormBinder;
    handleValueChanged(props: ITimeInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
export declare class CalendarInputFormBinder extends FormBinderBase<ICalendarInputProps, string, string> {
    constructor(dataPath: string);
    static customValue(dataName: string): CalendarInputFormBinder;
    handleValueChanged(props: ICalendarInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
export declare class AutoCompleteFormBinder implements IFormBinder<IAutoCompleteInputProps, any> {
    dataPath: string;
    private getItemFromId;
    constructor(dataPath: string, getItemFromId?: (id: string) => IAutoCompleteOption);
    setElementProperty(props: IAutoCompleteInputProps, dataBinder: IDataBinder<any>): void;
    handleValueChanged(props: IAutoCompleteInputProps, dataBinder: IDataBinder<any>, notifyChanged: () => void): void;
}
/** Form Binder helpers */
export declare class FormBinder {
    /** bind a custom form binder */
    custom<P>(formBinder: IFormBinder<P, any>): IFormBinderInjector<P>;
    /** bind to a 'hidden' input */
    hidden<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a string property to a 'password' input */
    password<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a string property to a 'text' input */
    text<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a string property to a 'email' input */
    textEmail<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    autoCompleteInput(dataName: string, getItemFromId?: (id: string) => IAutoCompleteOption): IFormBinderInjector<IAutoCompleteInputProps>;
    /** bind a 'date' string property to a CalendarInput (e.g. YYYY-MM-DD) */
    calendarInput(dataName: string): IFormBinderInjector<ICalendarInputProps>;
    /** bind a 'date' string property to a DateInput (e.g. YYYY-MM-DD) */
    dateInput(dataName: string): IFormBinderInjector<IDateInputProps>;
    /** bind a 'time' string property to a TimeInput (e.g. HH:MM) */
    timeInput(dataName: string): IFormBinderInjector<ITimeInputProps>;
    private defaultInputFormBinder<TDataPropValue, TTo>(dataName, type, valueConverter?, propertySet?);
    /** bind a number property to a range */
    range(dataName: string, options?: INumericOptions): IFormBinderInjector<React.DOMAttributes<any>>;
    /** uncontrolled text input */
    defaultText<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a number property to a 'text' input */
    textNumeric(dataName: string, options?: INumericOptions): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a TDataPropValue property to a select */
    selectCustom<TDataPropValue>(dataName: string, valueConverter?: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a string property to a select */
    select(dataName: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a number property to a select */
    selectNumeric(dataName: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a TDataPropValue[] property to a multi select */
    selectMultipleCustom<TDataPropValue>(dataName: string, valueConverter?: IValueConverter<TDataPropValue, string[]>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a string[] property to a multi select */
    selectMultiple(dataName: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a number[] property to a multi select */
    selectMultipleNumeric(dataName: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a TDataPropValue property to a 'checkbox' input */
    checkboxCustom<TDataPropValue>(dataName: string, valueConverter?: IValueConverter<TDataPropValue, boolean>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a boolean property to a 'checkbox' input */
    checkbox(dataName: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a TDataPropValue property to a 'radio' input, using trueValue and falseValue equality testing */
    checkboxConvert<TDataPropValue>(dataName: string, trueValue: TDataPropValue, falseValue: TDataPropValue): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a TDataPropValue property to a 'radio' input */
    radioCustom<TDataPropValue>(dataName: string, value: string, valueConverter: IInputValueConverter<TDataPropValue>): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a string property to a 'radio' input */
    radio(dataName: string, value: string): IFormBinderInjector<React.DOMAttributes<any>>;
    /** bind a number property to a 'radio' input */
    radioNumeric(dataName: string, value: number): IFormBinderInjector<React.DOMAttributes<any>>;
}
